package cat.ximpleries.beerradar;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.location.Location;
import android.location.LocationManager;
import android.os.Bundle;
import android.provider.Settings;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBar;
import android.support.v7.app.ActionBarActivity;
import android.text.Html;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.RatingBar;
import android.widget.SeekBar;
import android.widget.TextView;

import java.io.IOException;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;


public class MainActivity extends ActionBarActivity
        implements NavigationDrawerFragment.NavigationDrawerCallbacks {

    public static final int POS_MENU_NOTICIES = 1;
    public static final int POS_MENU_CERVESES = 2;
    public static final int POS_MENU_LOCALS = 3;
    public static final int POS_MENU_PUBLICACIONS = 4;
    public static final int POS_MENU_PERFIL = 5;
    public static ArrayList<NewsModel> newsItems = new ArrayList<>();
    public static ArrayList<BeerModel> beerItems = new ArrayList<>();
    public static ArrayList<CommentModel> commentsItems = new ArrayList<>();
    public static BeerAdapter beerAdapter;
    public static NewsAdapter newsAdapter;
    public static CommentsAdapter commentsAdapter;
    static String android_id = "";
    private static Updater updater;
    private static long newsUpdatedTimestamp = 0; // When the news were last updated
    private static long newsRefreshMs = TimeUnit.MINUTES.toMillis(30); // Milliseconds to "cache" the news
    private static int beerDbId;
    private static String beerRate;
    /**
     * Fragment managing the behaviors, interactions and presentation of the navigation drawer.
     */
    private NavigationDrawerFragment mNavigationDrawerFragment;
    /**
     * Used to store the last screen title. For use in {@link #restoreActionBar()}.
     */
    private CharSequence mTitle;

    private Profile profile = new Profile(this);

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        mNavigationDrawerFragment = (NavigationDrawerFragment)
                getSupportFragmentManager().findFragmentById(R.id.navigation_drawer);
        mTitle = getTitle();

        // Set up the drawer.
        mNavigationDrawerFragment.setUp(
                R.id.navigation_drawer,
                (DrawerLayout) findViewById(R.id.drawer_layout));

        updater = Updater.getInstance(this);
        updater.updateVenues(42, 2, VenueFragment.getInstance().getVenues());
        beerAdapter = new BeerAdapter(this, beerItems);
        newsAdapter = new NewsAdapter(this, newsItems);
        commentsAdapter = new CommentsAdapter(this, commentsItems);
        updater.updateNews(newsAdapter, newsItems);
        updater.updateBeers(beerAdapter, beerItems);
        updater.updateComments(commentsAdapter, commentsItems);
        onNavigationDrawerItemSelected(0);

        android_id = Settings.Secure.getString(getBaseContext().getContentResolver(),
                Settings.Secure.ANDROID_ID);
        updater.getProfile(android_id, profile);
    }

    @Override
    public void onNavigationDrawerItemSelected(int position) {
        // update the main content by replacing fragments
        FragmentManager fragmentManager = getSupportFragmentManager();
        if (position + 1 == POS_MENU_LOCALS) {
            mTitle = getString(R.string.menu_locals);
            VenueFragment venueFragment = VenueFragment.getInstance();
            LocationManager locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
            venueFragment.requestLocationUpdates(locationManager);
            fragmentManager.beginTransaction()
                    .replace(R.id.container, venueFragment)
                    .commit();
        } else {
            fragmentManager.beginTransaction()
                    .replace(R.id.container, PlaceholderFragment.newInstance(position + 1, PlaceholderFragment.TYPE.MAIN_SECTION))
                    .commit();
        }
    }

    public void displayVenueDetails(String name) {
        int idx = -1;
        ArrayList<VenueModel> venues = VenueFragment.getInstance().getVenues();
        for (int i = 0; i < venues.size(); i++) {
            if (venues.get(i).getNom().equals(name)) {
                idx = i;
                break;
            }
        }
        FragmentManager fragmentManager = getSupportFragmentManager();
        PlaceholderFragment fragment = PlaceholderFragment.newInstance(idx, PlaceholderFragment.TYPE.VENUE);
        if (!fragment.errorOccurred) {
            fragmentManager.beginTransaction()
                    .replace(R.id.container, fragment)
                    .commit();
        }
    }

    public void displayNewsDetails(int idx) {
        FragmentManager fragmentManager = getSupportFragmentManager();
        fragmentManager.beginTransaction()
                .add(PlaceholderFragment.newInstance(1, PlaceholderFragment.TYPE.MAIN_SECTION), "noticies")
                .addToBackStack("noticies")
                .replace(R.id.container, PlaceholderFragment.newInstance(idx, PlaceholderFragment.TYPE.NEWS))
                .commit();
    }

    public void displayBeerDetails(int beerId) {
        FragmentManager fragmentManager = getSupportFragmentManager();
        fragmentManager.beginTransaction()
                .replace(R.id.container, PlaceholderFragment.newInstance(beerId, PlaceholderFragment.TYPE.BEER))
                .commit();
    }

    public void onSectionAttached(int number) {
        switch (number) {
            case POS_MENU_NOTICIES:
                mTitle = getString(R.string.menu_noticies);
                break;
            case POS_MENU_CERVESES:
                mTitle = getString(R.string.menu_cerveses);
                break;
            case POS_MENU_LOCALS:
                mTitle = getString(R.string.menu_locals);
                break;
            case POS_MENU_PUBLICACIONS:
                mTitle = getString(R.string.menu_publicacions);
                break;
            case POS_MENU_PERFIL:
                mTitle = getString(R.string.menu_perfil);
                break;
        }
    }

    public void restoreActionBar() {
        ActionBar actionBar = getSupportActionBar();
        actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_STANDARD);
        actionBar.setDisplayShowTitleEnabled(true);
        actionBar.setTitle(mTitle);
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        if (!mNavigationDrawerFragment.isDrawerOpen()) {
            // Only show items in the action bar relevant to this screen
            // if the drawer is not showing. Otherwise, let the drawer
            // decide what to show in the action bar.
            getMenuInflater().inflate(R.menu.menu_main, menu);
            restoreActionBar();
            return true;
        }
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();
        return super.onOptionsItemSelected(item);
    }

    public void sendBeerRate(View view) {
        updater.updateRating((RatingBar) findViewById(R.id.ratingBar), android_id, beerDbId, Float.valueOf(beerRate.replace(",",".")));
    }

    public void getListPreferredBeers(View view) {
        beerItems.clear();
        beerAdapter.notifyDataSetChanged();
        updater.updateBeers(beerAdapter, beerItems, 2, 0, 0, android_id);
    }

    public void getListNearestBeers(View view) {
        Location loc = VenueFragment.getInstance().getLoc();
        double lat, lng;
        if (loc != null) {
            lat = loc.getLatitude();
            lng = loc.getLongitude();
        } else {
            lat = 41.3850639000000000;
            lng = 2.1734034999999494;
        }
        beerItems.clear();
        beerAdapter.notifyDataSetChanged();
        Updater.getInstance(this).updateBeers(beerAdapter, beerItems, 1, lat, lng, "");
    }

    public void getListRandomBeers(View view) {
        beerItems.clear();
        beerAdapter.notifyDataSetChanged();
        Updater.getInstance(this).updateBeers(beerAdapter, beerItems);
    }

    public void showAvatars(View view) {
        // Is the button now checked?
        boolean checked = ((RadioButton) view).isChecked();

        // Check which radio button was clicked
        switch (view.getId()) {
            case R.id.profileSelectMales:
                if (checked) {
                    findViewById(R.id.profileOptionsMale).setVisibility(View.VISIBLE);
                    findViewById(R.id.profileOptionsFemale).setVisibility(View.INVISIBLE);
                } else {
                    findViewById(R.id.profileOptionsMale).setVisibility(View.INVISIBLE);
                    findViewById(R.id.profileOptionsFemale).setVisibility(View.VISIBLE);
                }
                break;
            case R.id.profileSelectFemales:
                if (checked) {
                    findViewById(R.id.profileOptionsMale).setVisibility(View.INVISIBLE);
                    findViewById(R.id.profileOptionsFemale).setVisibility(View.VISIBLE);
                } else {
                    findViewById(R.id.profileOptionsMale).setVisibility(View.VISIBLE);
                    findViewById(R.id.profileOptionsFemale).setVisibility(View.INVISIBLE);
                }
                break;
        }
    }

    public void selectAvatar(View v) {
        profile.setSelectedAvatar(profile.buttonIdToAvatarId(v.getId()));
    }

    public void guardarPerfil(View v) {
        updater.saveProfile(android_id, profile.getSelectedAvatar(),
                ((TextView) findViewById(R.id.profileName)).getText().toString(),
                profile,
                (ImageView) findViewById(R.id.currentAvatar),
                this.getApplicationContext());
    }

    public void comenta(View v) {
        String comment = ((EditText) findViewById(R.id.nouComentari)).getText().toString();
        updater.sendComment(android_id, comment, beerDbId, getApplicationContext());
    }

    public static class PlaceholderFragment extends Fragment {

        private static final String ARG_SECTION_NUMBER = "section_number";
        public static boolean errorOccurred = false;
        private static TYPE mType;
        ListView list;

        public PlaceholderFragment() {
        }

        /**
         * Returns a new instance of this fragment for the given section
         * number.
         */
        public static PlaceholderFragment newInstance(int sectionNumber, TYPE type) {
            PlaceholderFragment fragment = new PlaceholderFragment();
            Bundle args = new Bundle();
            args.putInt(ARG_SECTION_NUMBER, sectionNumber);
            fragment.setArguments(args);
            mType = type;
            errorOccurred = false;
            return fragment;
        }

        @Override
        public View onCreateView(LayoutInflater inflater, ViewGroup container,
                                 Bundle savedInstanceState) {
            View rootView = null;
            if (mType.equals(TYPE.MAIN_SECTION)) {
                switch ((int) this.getArguments().get(ARG_SECTION_NUMBER)) {
                    case POS_MENU_NOTICIES:
                        rootView = inflater.inflate(R.layout.fragment_noticies, container, false);
                        list = (ListView) (rootView.findViewById(R.id.newslist));
                        list.setAdapter(newsAdapter);
                        newsAdapter.notifyDataSetChanged();
                        updater.updateNews(newsAdapter, newsItems);
                        break;
                    case POS_MENU_CERVESES:
                        rootView = inflater.inflate(R.layout.fragment_cerveses, container, false);
                        list = (ListView) (rootView.findViewById(R.id.beerlist));
                        list.setAdapter(beerAdapter);
                        updater.updateBeers(beerAdapter, beerItems);
                        break;
                    case POS_MENU_PUBLICACIONS:
                        updater.updateComments(commentsAdapter, commentsItems);
                        rootView = inflater.inflate(R.layout.fragment_publicacions, container, false);
                        list = (ListView) (rootView.findViewById(R.id.commentsList));
                        list.setAdapter(commentsAdapter);
                        break;
                    case POS_MENU_PERFIL:
                        Profile p = ((MainActivity) getActivity()).profile;
                        rootView = inflater.inflate(R.layout.fragment_perfil, container, false);
                        ImageView currentAvatar = (ImageView) rootView.findViewById(R.id.currentAvatar);
                        EditText pName = (EditText) rootView.findViewById(R.id.profileName);
                        currentAvatar.setImageDrawable(p.getAvatar());
                        pName.setText(p.getUserName());
                        TextView comentaris = (TextView) rootView.findViewById(R.id.profileCommments);
                        TextView puntuacions = (TextView) rootView.findViewById(R.id.profilePuntuacions);
                        updater.updateProfile(android_id, currentAvatar, pName, p);
                        updater.updateProfileCounters(android_id, comentaris, puntuacions);
                        break;
                    default:
                        rootView = inflater.inflate(R.layout.fragment_main, container, false);
                }
            } else if (mType.equals(TYPE.NEWS)) {
                int newsId = (int) this.getArguments().get(ARG_SECTION_NUMBER);
                NewsModel nm = newsItems.get(newsId);
                rootView = inflater.inflate(R.layout.fragment_noticies_detall, container, false);
                TextView ndTitle = (TextView) rootView.findViewById(R.id.ndTitle);
                ndTitle.setText(nm.getTitle());
                TextView ndDate = (TextView) rootView.findViewById(R.id.ndDate);
                ndDate.setText(nm.getDate());
                TextView ndContent = (TextView) rootView.findViewById(R.id.ndContent);
                ndContent.setText(nm.getText());
                TextView ndSource = (TextView) rootView.findViewById(R.id.ndSource);
                ndSource.setText(""); // TODO : put appropiate source e.g. NacióDigital, etc.
                ImageView ndImage = (ImageView) rootView.findViewById(R.id.ndImage);
                if (Updater.getInstance(this.getActivity()).bitmaps.containsKey(nm.getImageUrl())) {
                    Bitmap b = Updater.getInstance(this.getActivity()).bitmaps.get(nm.getImageUrl());
                    ndImage.setImageBitmap(b);
                    ndImage.setMinimumWidth(250);
                    ndImage.setMaxWidth(250);
                    ndImage.setMinimumHeight(250);
                    ndImage.setMaxHeight(250);
                } else {
                    new Updater.DownloadImageTask(ndImage, 250).execute(nm.getImageUrl());
                }
            } else if (mType.equals(TYPE.BEER)) {
                int beerId = (int) this.getArguments().get(ARG_SECTION_NUMBER);
                BeerModel beer = null;
                for (BeerModel b : beerItems) {
                    if (b.getIdCervesa() == beerId) {
                        beer = b;
                        break;
                    }
                }

                if (beer == null) {
                    try {
                        beer = Updater.getInstance(getActivity()).getBeerSynchronously(beerId);
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }

                rootView = inflater.inflate(R.layout.fragment_cerveses_detall, container, false);

                if (beer != null) {
                    beerDbId = beer.getIdCervesa();

                    TextView beerName = (TextView) rootView.findViewById(R.id.titol_local);
                    beerName.setText(beer.getNom());
                    TextView beerType = (TextView) rootView.findViewById(R.id.tipus);
                    beerType.setText(beer.getEstil());
                    SeekBar userBeerRating = (SeekBar) rootView.findViewById(R.id.seekBar);
                    final TextView label = (TextView) rootView.findViewById(R.id.seekBarNum);
                    SeekBar.OnSeekBarChangeListener seekBarListener = new SeekBar.OnSeekBarChangeListener() {
                        @Override
                        public void onStopTrackingTouch(SeekBar seekBar) {
                            beerRate = label.getText().toString();
                        }

                        @Override
                        public void onStartTrackingTouch(SeekBar seekBar) {
                        }

                        @Override
                        public void onProgressChanged(SeekBar seekBar, int progress,
                                                      boolean fromUser) {
                            label.setText(String.format("%.2f", 5.0 * progress / 100));
                        }
                    };

                    userBeerRating.setOnSeekBarChangeListener(seekBarListener);
                    RatingBar averageBeerRating = (RatingBar) rootView.findViewById(R.id.ratingBar);
                    averageBeerRating.setRating(beer.getPuntuacioMitja());

                    String beerImageUrl = beer.getImageUrl();
                    ImageView beerImage = (ImageView) rootView.findViewById(R.id.barDetailImage);
                    if (Updater.getInstance(this.getActivity()).bitmaps.containsKey(beerImageUrl)) {
                        Bitmap b = Updater.getInstance(this.getActivity()).bitmaps.get(beerImageUrl);
                        beerImage.setImageBitmap(Updater.getScaledBitmap(b,250));
                    } else {
                        new Updater.DownloadImageTask(beerImage, 250).execute(beerImageUrl);
                    }

                    // Demanar quin és el local més proper on tenen aquesta cervesa
                    Location loc = VenueFragment.getInstance().getLoc();
                    double lat, lng;
                    if (loc != null) {
                        lat = loc.getLatitude();
                        lng = loc.getLongitude();
                    } else {
                        lat = 41.3850639000000000;
                        lng = 2.1734034999999494;
                    }
                    final TextView venueWithBeer = (TextView) rootView.findViewById(R.id.cervesa_popular);
                    venueWithBeer.setOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View v) {
                            ((MainActivity) getActivity()).displayVenueDetails(venueWithBeer.getText().toString());
                        }
                    });

                    Updater.getInstance(this.getActivity()).searchVenueWithBeer(beer.getIdCervesa(), lat, lng, venueWithBeer);
                }

            } else if (mType.equals(TYPE.VENUE)) {
                int idx = (int) this.getArguments().get(ARG_SECTION_NUMBER);
                rootView = inflater.inflate(R.layout.fragment_locals_detall, container, false);

                if (VenueFragment.getInstance().getVenues().size() > idx && idx > 0) {
                    errorOccurred = false;
                    VenueModel venue = VenueFragment.getInstance().getVenues().get(idx);
                    TextView venueName = (TextView) rootView.findViewById(R.id.titol_local);
                    venueName.setText(venue.getNom());
                    TextView venueType = (TextView) rootView.findViewById(R.id.tipusBar);
                    venueType.setText(VenueModel.tipusStr[venue.getTipus()]);
                    TextView adreca = (TextView) rootView.findViewById(R.id.adreca);
                    adreca.setText(Html.fromHtml(venue.getAdreca()));

                    Updater u = Updater.getInstance(this.getActivity());
                    final TextView popularBeer = (TextView) rootView.findViewById(R.id.cervesa_popular);
                    final TextView preferredBeer = (TextView) rootView.findViewById(R.id.cervesa_preferida);
                    final TextView suggestedBeer = (TextView) rootView.findViewById(R.id.cervesa_suggerida);

                    u.getMostPopularBeer(venue.getId(), popularBeer);
                    u.getPreferredBeer(venue.getId(), android_id, preferredBeer);
                    u.getSuggestedBeer(venue.getId(), android_id, suggestedBeer);

                    popularBeer.setOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View v) {
                            CharSequence hint = popularBeer.getHint();
                            if (hint != null && !hint.toString().isEmpty()) {
                                int beerId = Integer.valueOf(hint.toString());
                                ((MainActivity) getActivity()).displayBeerDetails(beerId);
                            }
                        }
                    });

                    preferredBeer.setOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View v) {
                            CharSequence hint = preferredBeer.getHint();
                            if (hint != null && !hint.toString().isEmpty()) {
                                int beerId = Integer.valueOf(hint.toString());
                                ((MainActivity) getActivity()).displayBeerDetails(beerId);
                            }
                        }
                    });

                    suggestedBeer.setOnClickListener(new View.OnClickListener() {
                        @Override
                        public void onClick(View v) {
                            CharSequence hint = suggestedBeer.getHint();
                            if (hint != null && !hint.toString().isEmpty()) {
                                int beerId = Integer.valueOf(hint.toString());
                                ((MainActivity) getActivity()).displayBeerDetails(beerId);
                            }
                        }
                    });

                    String imgUrl = venue.getImatgeUrl();
                    ImageView venueImage = (ImageView) rootView.findViewById(R.id.barDetailImage);
                    if (Updater.getInstance(this.getActivity()).bitmaps.containsKey(imgUrl)) {
                        Bitmap b = Updater.getInstance(this.getActivity()).bitmaps.get(imgUrl);
                        venueImage.setImageBitmap(Updater.getScaledBitmap(b,250));
                    } else {
                        new Updater.DownloadImageTask(venueImage, 250).execute(imgUrl);
                    }
                } else {
                    errorOccurred = true;
                }
            }

            return rootView;
        }

        @Override
        public void onAttach(Activity activity) {
            super.onAttach(activity);
            ((MainActivity) activity).onSectionAttached(
                    getArguments().getInt(ARG_SECTION_NUMBER));
        }

        public static enum TYPE {MAIN_SECTION, NEWS, BEER, VENUE}
    }
}
