package cat.ximpleries.beerradar;

import android.app.Activity;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.AsyncTask;
import android.util.Log;
import android.widget.ImageView;
import android.widget.RatingBar;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.MarkerOptions;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;

public class Updater {

    public static HashMap<String, Bitmap> bitmaps = new HashMap<>();
    static Activity parentActivity;
    private static volatile Updater instance;
    private String synchRes;

    private Updater() {
    }

    public static Updater getInstance(Activity activity) {
        if (instance == null) {
            synchronized (Updater.class) {
                if (instance == null) {
                    instance = new Updater();
                }
            }
        }
        if(activity!=null)
            parentActivity = activity;

        return instance;
    }

    public static int[] getMaxDimensions(Bitmap b, int max) {
        int[] newWH = {0, 0};
        float proportion = (float) b.getWidth() / (float) b.getHeight();
        newWH[0] = max;
        newWH[1] = Math.round(max / proportion);
        return newWH;
    }

    public static Bitmap getScaledBitmap (Bitmap b, int maxSize) {
        int[] newWH = getMaxDimensions(b, maxSize);
        return Bitmap.createScaledBitmap(b, newWH[0], Math.min(newWH[1], 300), true);
    }

    public void updateBeers(BeerAdapter adapter, ArrayList<BeerModel> beers) {
        new DownloadBeers(adapter, beers, 0, 0, 0, "");
    }

    public void updateBeers(BeerAdapter adapter, ArrayList<BeerModel> beers, int listType, double lat, double lng, String idUser) {
        new DownloadBeers(adapter, beers, listType, lat, lng, idUser);
    }

    public void updateNews(NewsAdapter adapter, ArrayList<NewsModel> news) {
        new DownloadNews(adapter, news);
    }

    public void updateComments(CommentsAdapter adapter, ArrayList<CommentModel> comments) {
        new DownloadComments(adapter, comments);
    }

    public void updateVenues(double lat, double lng, ArrayList<VenueModel> venues) {
        // Only download once, as currently all venues are downloaded
        // Otherwise, to adapt the venues to latitude / longitude, we should clear the ArrayList before
        if(venues.size() > 0)
            return;

        new DownloadVenues(lat, lng, venues);
    }

    public void updateVenues(double lat, double lng, ArrayList<VenueModel> venues, GoogleMap map) {
        new DownloadVenues(lat, lng, venues, map);
    }

    public void updateRating(RatingBar rb, String userId, int beerId, float rating) {
        new RateBeer(rb, userId, beerId, rating);
    }

    public void searchVenueWithBeer(int idCervesa, double lat, double lng, TextView venueWithBeer) {
        new GetVenue(idCervesa, lat, lng, venueWithBeer);
    }

    public void getMostPopularBeer(int idLocal, TextView tv) {
        new GetInterestingBeerInVenue("http://151.80.154.70/beerradar/service.php?req=bar_most_popular_beer&params=" + idLocal, tv);
    }

    public void getPreferredBeer(int idLocal, String userId, TextView tv) {
        new GetInterestingBeerInVenue("http://151.80.154.70/beerradar/service.php?req=beer_most_preferred&params=" + idLocal + ";" + userId, tv);
    }

    public void getSuggestedBeer(int idLocal, String userId, TextView tv) {
        new GetInterestingBeerInVenue("http://151.80.154.70/beerradar/service.php?req=beer_suggestion&params=" + idLocal + ";" + userId, tv);
    }

    public void getProfile(String userId, Profile profile) {
        new GetProfile(userId, profile);
    }

    public BeerModel getBeerSynchronously(int idCervesa) throws IOException {
        BeerModel n = null;
        String url = "http://151.80.154.70/beerradar/service.php?req=get_single_beer&params=" + idCervesa;

        SynchDownload b = new SynchDownload(url);
        b.start();

        synchronized (b) {
            try {
                Log.d("Updater", "Waiting for b to complete...");
                b.wait();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }

        if (!synchRes.isEmpty()) {
            JSONObject response = null;
            try {
                response = new JSONObject(synchRes);
                JSONArray jArr = response.getJSONArray("res");
                for (int i = 0; i < jArr.length(); i++) {
                    n = new BeerModel();
                    JSONObject obj = jArr.getJSONObject(i);
                    n.setIdCervesa(obj.getInt("idcervesa"));
                    n.setNom(obj.getString("nom"));
                    n.setEstil(obj.getString("nom_estil"));
                    n.setLlocOrigen(obj.getString("lloc_origen"));
                    n.setCerveseria(obj.getString("cerveseria"));
                    n.setImageUrl(obj.getString("imatge"));
                    n.setPuntuacioMitja((float) obj.getDouble("mitja"));
                    return n;
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
        return n;
    }

    // Given a URL, establishes an HttpUrlConnection and retrieves
    // the web page content as a InputStream, which it returns as
    // a string.
    private String downloadUrl(String myurl) throws IOException {
        InputStream is = null;
        // Only display the first 500 characters of the retrieved
        // web page content.
        int len = 500;

        try {
            URL url = new URL(myurl);
            HttpURLConnection conn = (HttpURLConnection) url.openConnection();
            conn.setReadTimeout(10000 /* milliseconds */);
            conn.setConnectTimeout(15000 /* milliseconds */);
            conn.setRequestMethod("GET");
            conn.setDoInput(true);
            // Starts the query
            conn.connect();
            int response = conn.getResponseCode();
            Log.d("HTTP", "The response is: " + response);
            is = conn.getInputStream();

            // Convert the InputStream into a string
            String contentAsString = readIt(is, len);
            return contentAsString;

            // Makes sure that the InputStream is closed after the app is
            // finished using it.
        } finally {
            if (is != null) {
                is.close();
            }
        }
    }

    // Reads an InputStream and converts it to a String.
    public String readIt(InputStream stream, int len) throws IOException, UnsupportedEncodingException {
        BufferedReader reader = new BufferedReader(new InputStreamReader(stream));
        StringBuilder out = new StringBuilder();
        String line;
        while ((line = reader.readLine()) != null) {
            out.append(line);
        }

        reader.close();
        return out.toString();
    }

    public void updateProfile(String userId, ImageView currentAvatar, TextView pName, Profile profile) {
        new GetProfile(userId, currentAvatar, pName, profile);
    }

    public void updateProfileCounters(String userId, TextView friends, TextView rates) {
        new UpdateProfileCounters(userId, friends, rates);
    }

    public void saveProfile(String userId, int avatarId, String nom, Profile profile, ImageView currentAvatar, Context context) {
        try {
            new SaveProfile(userId, avatarId, nom, profile, currentAvatar, context);
        } catch (Exception e) {
        }
    }

    public void sendComment(String userId, String comment, int beerId, Context context) {
        try {
            new SendComment(userId, comment, beerId, context);
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
    }

    public static class DownloadImageTask extends AsyncTask<String, Void, Bitmap> {

        private String mUrl;
        private String fullUrl;
        private ImageView mImgvw;
        private int maxSize;

        DownloadImageTask(ImageView imgvw, int maxSize) {
            mImgvw = imgvw;
            this.maxSize = maxSize;
        }

        protected Bitmap doInBackground(String... urls) {
            mUrl = urls[0];
            Bitmap mIcon11 = null;
            try {
                fullUrl = (!mUrl.contains("http")) ? "http://151.80.154.70/beerradar/imgs/" + mUrl : mUrl;
                InputStream in = new java.net.URL(fullUrl).openStream();
                mIcon11 = BitmapFactory.decodeStream(in);
            } catch (Exception e) {
                Log.e("Error", e.getMessage());
                e.printStackTrace();
            }
            return mIcon11;
        }

        protected void onPostExecute(Bitmap result) {
            if (result == null)
                return;

            int[] newWH = getMaxDimensions(result, maxSize);
            Bitmap scaled = Bitmap.createScaledBitmap(result, newWH[0], Math.min(newWH[1], 300), true);
            Updater.bitmaps.put(mUrl, result);
            mImgvw.setImageBitmap(scaled);
        }
    }

    private class RateBeer extends DownloadWebpageTask {
        RatingBar ratingBar;

        RateBeer(RatingBar rb, String userId, int beerId, float rating) {
            super("http://151.80.154.70/beerradar/service.php?upd=puntua&params=" + userId + ";" + beerId + ";" + rating);
            ratingBar = rb;
        }

        @Override
        protected void onPostExecute(String result) {
            try {
                ratingBar.setRating(Float.valueOf(result.replace(",",".")));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private class SendComment extends DownloadWebpageTask {
        Context mcontext;

        SendComment(String userId, String comment, int beerId, Context context) throws UnsupportedEncodingException {
            super("http://151.80.154.70/beerradar/service.php?upd=send_comment&params=" + userId + ";" + URLEncoder.encode(comment, "UTF-8") + ";" + beerId);
            mcontext = context;
        }

        @Override
        protected void onPostExecute(String result) {
            CharSequence text = "Enviat! (visible a 'Publicacions')";
            int duration = Toast.LENGTH_SHORT;

            Toast toast = Toast.makeText(mcontext, text, duration);
            toast.show();
        }
    }

    private class DownloadComments extends DownloadWebpageTask {
        CommentsAdapter adapter;
        ArrayList<CommentModel> comments;

        DownloadComments(CommentsAdapter adapter, ArrayList<CommentModel> comments) {
            super("http://151.80.154.70/beerradar/service.php?req=get_comments");
            this.adapter = adapter;
            this.comments = comments;
        }

        @Override
        protected void onPostExecute(String result) {
            try {
                JSONObject response = new JSONObject(result);
                JSONArray jArr = response.getJSONArray("res");
                comments.clear();
                for (int i = 0; i < jArr.length(); i++) {
                    final CommentModel n = new CommentModel();
                    JSONObject obj = jArr.getJSONObject(i);
                    n.setComentari(obj.getString("comentari").replace("\\", ""));
                    n.setData(obj.getString("data"));
                    n.setNomCervesa(obj.getString("nom"));
                    n.setNom_usuari(obj.getString("nom_usuari"));
                    n.setAvatar_id(obj.getInt("avatar_id"));
                    comments.add(n);
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }
            adapter.notifyDataSetChanged();
        }
    }


    private class UpdateProfileCounters extends DownloadWebpageTask {
        TextView comments;
        TextView rates;

        UpdateProfileCounters(String userId, TextView comments, TextView rates) {
            super("http://151.80.154.70/beerradar/service.php?req=profile_counters&params=" + userId);
            this.comments = comments;
            this.rates = rates;
        }

        @Override
        protected void onPostExecute(String result) {
            try {
                JSONObject response = new JSONObject(result);
                JSONArray jArr = response.getJSONArray("res");
                if (jArr != null) {
                    JSONObject obj = jArr.getJSONObject(0);
                    rates.setText(obj.getInt("num_puntuacions") + " cerveses");
                    comments.setText(obj.getInt("num_comentaris") + " comentaris");
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    }


    private class SaveProfile extends DownloadWebpageTask {
        Profile mProfile;
        String mUserId;
        Context mcontext;
        ImageView mCurrentAvatar;

        SaveProfile(String userId, int avatarId, String nom, Profile profile, ImageView currentAvatar, Context context) throws UnsupportedEncodingException {
            super("http://151.80.154.70/beerradar/service.php?upd=profile&params=" + userId + ";" + avatarId + ";" + URLEncoder.encode(nom, "UTF-8"));
            mProfile = profile;
            mUserId = userId;
            mcontext = context;
            mCurrentAvatar = currentAvatar;
        }

        @Override
        protected void onPostExecute(String result) {
            try {
                JSONObject response = new JSONObject(result);
                JSONArray jArr = response.getJSONArray("res");
                if (jArr != null) {
                    JSONObject obj = jArr.getJSONObject(0);
                    mProfile.setUserName(obj.getString("nom_usuari"));
                    mProfile.setUserAvatar(obj.getInt("avatar_id"));
                    mCurrentAvatar.setImageDrawable(mProfile.getAvatar());
                } else {
                    mProfile.setUserName(mUserId);
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }
            CharSequence text = "Guardat!";
            int duration = Toast.LENGTH_SHORT;

            Toast toast = Toast.makeText(mcontext, text, duration);
            toast.show();
        }
    }

    private class GetProfile extends DownloadWebpageTask {
        Profile mProfile;
        String mUserId;
        ImageView mCurrentAvatar;
        TextView mCurrentName;

        GetProfile(String userId, Profile profile) {
            this(userId, null, null, profile);
        }

        GetProfile(String userId, ImageView currentAvatar, TextView currentName, Profile profile) {
            super("http://151.80.154.70/beerradar/service.php?req=get_profile&params=" + userId);
            mProfile = profile;
            mUserId = userId;
            mCurrentAvatar = currentAvatar;
            mCurrentName = currentName;
        }

        @Override
        protected void onPostExecute(String result) {
            try {
                JSONObject response = new JSONObject(result);
                JSONArray jArr = response.getJSONArray("res");
                if (jArr != null) {
                    JSONObject obj = jArr.getJSONObject(0);
                    mProfile.setUserName(obj.getString("nom_usuari"));
                    mProfile.setUserAvatar(obj.getInt("avatar_id"));
                    mProfile.setSelectedAvatar(obj.getInt("avatar_id"));
                    if (mCurrentName != null) {
                        mCurrentName.setText(mProfile.getUserName());
                    }
                    if (mCurrentAvatar != null) {
                        mCurrentAvatar.setImageDrawable(mProfile.getAvatar());
                    }
                } else {
                    mProfile.setUserName(mUserId);
                    mProfile.setUserAvatar(0);
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    }

    private class GetVenue extends DownloadWebpageTask {
        TextView tv;

        GetVenue(int idCervesa, double lat, double lng, TextView venueWithBeer) {
            super("http://151.80.154.70/beerradar/service.php?req=bar_with_beer&params=" + idCervesa + ";" + lat + ";" + lng);
            this.tv = venueWithBeer;
        }

        @Override
        protected void onPostExecute(String result) {
            try {
                JSONObject response = new JSONObject(result);
                JSONArray jArr = response.getJSONArray("res");

                JSONObject obj = jArr.getJSONObject(0);
                tv.setText(obj.getString("nom"));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    }

    private class GetInterestingBeerInVenue extends DownloadWebpageTask {
        TextView tv;

        GetInterestingBeerInVenue(String url, TextView fieldToUpdate) {
            super(url);
            this.tv = fieldToUpdate;
        }

        @Override
        protected void onPostExecute(String result) {
            try {
                JSONObject response = new JSONObject(result);
                JSONArray jArr = response.getJSONArray("res");
                JSONObject obj = jArr.getJSONObject(0);
                tv.setText(obj.getString("nom"));
                tv.setHint(obj.getString("idcervesa"));
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    }

    private class DownloadVenues extends DownloadWebpageTask {
        NewsAdapter adapter;
        ArrayList<VenueModel> venues;
        GoogleMap map;

        DownloadVenues(double lat, double lng, ArrayList<VenueModel> venues) {
            super("http://151.80.154.70/beerradar/service.php?req=bars_nearby&params=" + lat + ";" + lng);
            this.venues = venues;
        }

        DownloadVenues(double lat, double lng, ArrayList<VenueModel> venues, GoogleMap map) {
            super("http://151.80.154.70/beerradar/service.php?req=bars_nearby&params=" + lat + ";" + lng);
            this.venues = venues;
            this.map = map;
        }

        @Override
        protected void onPostExecute(String result) {
            try {
                JSONObject response = new JSONObject(result);
                JSONArray jArr = response.getJSONArray("res");
                for (int i = 0; i < jArr.length(); i++) {
                    final VenueModel n = new VenueModel();
                    JSONObject obj = jArr.getJSONObject(i);
                    n.setId(obj.getInt("idlocal"));
                    n.setNom(obj.getString("nom"));
                    n.setTipus(obj.getInt("tipus"));
                    n.setAdreca(obj.getString("adreca"));
                    n.setTelefon(obj.getString("telf"));
                    n.setImatgeUrl(obj.getString("imatge"));
                    n.setLatitud(obj.getDouble("latitud"));
                    n.setLongitud(obj.getDouble("longitud"));
                    venues.add(n);
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }

            if (map != null) {
                for (VenueModel v : venues) {
                    map.addMarker(new MarkerOptions()
                            .icon(BitmapDescriptorFactory.fromResource(R.drawable.beer_icon))
                            .anchor(0.0f, 1.0f) // Anchors the marker on the bottom left
                            .position(new LatLng(v.getLatitud(), v.getLongitud())));
                }
            }
        }
    }

    private class DownloadNews extends DownloadWebpageTask {
        NewsAdapter adapter;
        ArrayList<NewsModel> news;

        DownloadNews(NewsAdapter adapter, ArrayList<NewsModel> news) {
            super("http://151.80.154.70/beerradar/service.php?req=noticies");
            this.adapter = adapter;
            this.news = news;
        }

        @Override
        protected void onPostExecute(String result) {
            try {
                JSONObject response = new JSONObject(result);
                JSONArray jArr = response.getJSONArray("res");
                for (int i = 0; i < jArr.length(); i++) {
                    final NewsModel n = new NewsModel();
                    JSONObject obj = jArr.getJSONObject(i);
                    n.setTitle(obj.getString("titol"));
                    n.setDate(obj.getString("data"));
                    n.setText(obj.getString("text"));
                    n.setImageUrl(obj.getString("img"));
                    news.add(n);
                }
                adapter.notifyDataSetChanged();

            } catch (JSONException e) {
                e.printStackTrace();
            }
        }
    }

    private class DownloadBeers extends DownloadWebpageTask {
        BeerAdapter adapter;
        ArrayList<BeerModel> beers;

        DownloadBeers(BeerAdapter adapter, ArrayList<BeerModel> beers, int listType, double lat, double lng, String iduser) {
            super("http://151.80.154.70/beerradar/service.php?req=cerveses&params=" + listType + ";" + lat + ";" + lng + ";" + iduser);
            this.adapter = adapter;
            this.beers = beers;
        }

        @Override
        protected void onPostExecute(String result) {
            try {
                JSONObject response = new JSONObject(result);
                JSONArray jArr = response.getJSONArray("res");
                for (int i = 0; i < jArr.length(); i++) {
                    final BeerModel n = new BeerModel();
                    JSONObject obj = jArr.getJSONObject(i);
                    n.setIdCervesa(obj.getInt("idcervesa"));
                    n.setNom(obj.getString("nom"));
                    n.setEstil(obj.getString("nom_estil"));
                    n.setLlocOrigen(obj.getString("lloc_origen"));
                    n.setCerveseria(obj.getString("cerveseria"));
                    n.setImageUrl(obj.getString("imatge"));
                    n.setPuntuacioMitja((float) obj.getDouble("mitja"));
                    beers.add(n);
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }
            adapter.notifyDataSetChanged();
        }
    }

    private class DownloadWebpageTask extends AsyncTask<String, Void, String> {
        DownloadWebpageTask(String url) {
            ConnectivityManager connMgr = (ConnectivityManager) (parentActivity.getSystemService(Context.CONNECTIVITY_SERVICE));
            NetworkInfo networkInfo = connMgr.getActiveNetworkInfo();
            if (networkInfo != null && networkInfo.isConnected()) {
                this.execute(url);
            } else {
                // "No network connection available."
            }
        }

        @Override
        protected String doInBackground(String... urls) {
            // params comes from the execute() call: params[0] is the url.
            try {
                return downloadUrl(urls[0]);
            } catch (IOException e) {
                return "Unable to retrieve web page. URL may be invalid.";
            }
        }
    }

    class SynchDownload extends Thread {
        String url;

        public SynchDownload(String url) {
            synchRes = "";
            this.url = url;
        }

        @Override
        public void run() {
            synchronized (this) {
                try {
                    synchRes = downloadUrl(url);
                } catch (IOException e) {
                    e.printStackTrace();
                }
                notify();
            }
        }
    }
}
