package cat.ximpleries.beerradar;

import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;

import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.SupportMapFragment;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;

import java.util.ArrayList;

public class VenueFragment extends Fragment implements OnMapReadyCallback {

    static private ArrayList<VenueModel> venues;
    static private VenueFragment fragment;
    private Location loc;
    private GoogleMap map;

    public VenueFragment() {
    }

    public static VenueFragment getInstance() {
        if (fragment == null) {
            fragment = new VenueFragment();
            venues = new ArrayList<>();
        }
        return fragment;
    }

    public Location getLoc() {
        return loc;
    }

    public void setLoc(Location loc) {
        this.loc = loc;
    }

    public GoogleMap getMap() {
        return map;
    }

    public void setMap(GoogleMap map) {
        this.map = map;
    }

    public ArrayList<VenueModel> getVenues() {
        return venues;
    }

    public void setVenues(ArrayList<VenueModel> venues) {
        this.venues = venues;
    }

    public void requestLocationUpdates(LocationManager locationManager) {
        // Define a listener that responds to location updates
        LocationListener locationListener = new LocationListener() {
            public void onLocationChanged(Location location) {
                Updater.getInstance(getActivity()).updateVenues(location.getLatitude(), location.getLongitude(), venues);
                loc = location;
            }

            public void onStatusChanged(String provider, int status, Bundle extras) {
            }

            public void onProviderEnabled(String provider) {
            }

            public void onProviderDisabled(String provider) {
            }
        };

        // Register the listener with the Location Manager to receive location updates
        locationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, locationListener);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        View rootView = inflater.inflate(R.layout.fragment_locals, container, false);

        SupportMapFragment mapFragment = (SupportMapFragment) getChildFragmentManager()
                .findFragmentById(R.id.map1);
        mapFragment.getMapAsync(this);

        return rootView;
    }

    @Override
    public void onMapReady(GoogleMap map) {
        this.map = map;
        LatLng latLng;
        double lat, lng;

        if (loc != null) {
            lat = loc.getLatitude();
            lng = loc.getLongitude();
        } else {
            lat = 41.3850639000000000;
            lng = 2.1734034999999494;
        }
        latLng = new LatLng(lat, lng);

        map.setMyLocationEnabled(true);
        map.moveCamera(CameraUpdateFactory.newLatLngZoom(latLng, 13));


        map.addMarker(new MarkerOptions()
                .position(latLng));
        map.setOnMarkerClickListener(new GoogleMap.OnMarkerClickListener() {
            @Override
            public boolean onMarkerClick(Marker marker) {
                ((MainActivity) getActivity()).displayVenueDetails(marker.getTitle());
                return true;
            }
        });
        if (venues.size() == 0) {
            Updater.getInstance(getActivity()).updateVenues(lat, lng, venues, map);
        } else {
            for (VenueModel v : venues) {
                map.addMarker(new MarkerOptions()
                        .icon(BitmapDescriptorFactory.fromResource(R.drawable.beer_icon))
                        .anchor(0.0f, 1.0f) // Anchors the marker on the bottom left
                        .position(new LatLng(v.getLatitud(), v.getLongitud()))
                        .title(v.getNom()));
            }
        }
    }

}
